#
#===============================================================================
#
#         FILE:  PQAFunctions.pm
#
#  DESCRIPTION:  This module contains the general functions required.,
#
#        FILES:  ---
#         BUGS:  ---
#        NOTES:  ---
#       AUTHOR:   (), <>
#      COMPANY:  
#      VERSION:  1.0
#      CREATED:  07/19/08 09:07:34 IST
#     REVISION:  ---
#===============================================================================

use strict 'vars';
use warnings;

use PQAGlobals;


#===  FUNCTION  ================================================================
#         NAME:  HandleOptions
#      PURPOSE:  To handle and store the options in global hash
#   PARAMETERS:  option variables
#      RETURNS:  exits when the required options are not there.,
#  DESCRIPTION:  checks for the options, and if it is not there exit, else
#  store the option values in the global hash.
#       THROWS:  no exceptions
#     COMMENTS:  none
#     SEE ALSO:  n/a
#===============================================================================

sub HandleOptions {
	my ( $t, $z, $f, $n ) = @_;

	#print "t->$t,z->$z,f->$f,n->$n";

	# -f is a compulsory option with argument.
					# FIXME: if the log file name is 1, then also it will die., but it should not.
	die "-f=<FILENAME> should be there\n" if not defined $f or $f eq 1;
	# if -n is given without number then alert it.
	warn "-n=<N> should be given, N should be greater than 1, default 20 is taken now\n" if defined $n and $n eq 1;
	
	# if -n option is given with number, then take it as it is.
	# 	if -n is not given, or -n with out number argument then take 20 as default.
	$n = 20 if ( (not defined $n) or ( $n =~ /\D/ ) ) ;

	#---------------------------------------------------------------------------
	# Get the options from the command line and store in PQAGlobals::options hash.
	#---------------------------------------------------------------------------
	$PQAGlobals::options{"f"}=$f;
	$PQAGlobals::options{"n"}=$n;
	$PQAGlobals::options{"z"}=$z;
	$PQAGlobals::options{"t"}=$t;

	return 1;
}	# ----------  end of subroutine HandleOptions  ----------


#===  FUNCTION  ================================================================
#         NAME:  CheckArg
#      PURPOSE:  check the number of arguments for the function
#   PARAMETERS:  
#   	1. all arguments(array), 
#   	2. number of arguments expected
#      RETURNS:  True if number of arguments are equal to expected arguments.,
#      Else, print WARNING, returns false.
#  DESCRIPTION:  
#  		1. take the last argument which will contain the number of argument expected,
#  		2. check it with the count of @_ array,
#		3. according to the result, print WARNING or return true.,
#       THROWS:  no exceptions
#     COMMENTS:  none
#     SEE ALSO:  n/a
#===============================================================================


sub CheckArg {
# pop the last argument	
	my $expected_arg = pop(@_);

# checking the number of arguments	

# if it equals return true	
	if ( @_ == $expected_arg )  {
		return 1;	
	} elsif ( @_ > $expected_arg ){
# if the number of arguments is more, print warning		
		print "WARNING: " . (caller(1))[3] . " excess number of arguments passed";
	} elsif ( @_ < $expected_arg ){
# if the number of arguments is less, print warning		
		print "WARNING: " . (caller(1))[3] . " less number of arguments passed";
	}

	return 0;
}	# ----------  end of subroutine CheckArg  ----------

1;
