#
#===============================================================================
#
#         FILE:  ReportGenerator.pm
#
#  DESCRIPTION:  This to generate the report ( HTML or text ) from the hash
#
#        FILES:  ---
#         BUGS:  ---
#        NOTES:  ---
#       AUTHOR:   (), <>
#      COMPANY:  
#      VERSION:  1.0
#      CREATED:  09/27/08 19:04:06 IST
#     REVISION:  ---
#===============================================================================

use strict;
use warnings;

use PQAGlobals;


#===  FUNCTION  ================================================================
#         NAME:  PrintOutput
#      PURPOSE:  print the report.
#   PARAMETERS:  nothing
#      RETURNS:  1
#  DESCRIPTION:  This calls the required functions one by one for print the report.
#       THROWS:  no exceptions
#     COMMENTS:  none
#     SEE ALSO:  n/a
#===============================================================================

sub PrintOutput {
	my ( @query_with_duration );
	# start print the report.
	startreport();
	# print most Frequent queries part
	PrintMostFreqQueries();

	# if all the queries does not have duration then there is no meaning to 
	# call the function QueriesTookMostTime 
	@query_with_duration = grep { $PQAGlobals::query_details{$_}->[4] > 0 } keys %PQAGlobals::query_details;
	 if ( @query_with_duration ) {
		# print Slowest Queries part
		SlowestQueries();
		# print the fastest queries
		FastestQueries();
		# print QueriesTookMostTime part
		QueriesTookMostTime();
	}

	if ( %PQAGlobals::error_query_details )  {	
		# print the errors in the report
		PrintErrorQuery();
	}

	# end the report.
	endreport();
	
	return 1;
}	# ----------  end of subroutine PrintOutput  ----------


#===  FUNCTION  ================================================================
#         NAME:  PrintMostFreqQueries
#      PURPOSE:  To print the most frequent queries
#   PARAMETERS:  nothing
#      RETURNS:  1
#  DESCRIPTION:  To print the most frequent queries 
#       THROWS:  no exceptions
#     COMMENTS:  none
#     SEE ALSO:  n/a
#===============================================================================
sub PrintMostFreqQueries {
	my @sorted;
	my $n_to_show;
	my $i = 1;
	
	# sort the hash key based up on the total count in it., and return it into array.
		# hashkey => [ total_count, .... ]
	@sorted = sort { $PQAGlobals::query_details{$b}->[0] <=> $PQAGlobals::query_details{$a}->[0] } keys %PQAGlobals::query_details;

	# remove the unwanted elements from the array, i.e show only the n number of elements.
	# if n is not given, then default is top 20.
	$n_to_show = $PQAGlobals::options{"n"};
	# splice only if the available number of elements is more than the required number of elements
 	splice(@sorted, $n_to_show)	if scalar(@sorted) > $n_to_show;

	if ($PQAGlobals::options{"t"})  {
		# print text report.
		map { print "$PQAGlobals::query_details{$_}->[0]\t$_\n"; }  @sorted;
	} else  {
		# print html report
		print "<BR><H3>Most Frequent queries</H3>";
		print '<TABLE BORDER="0">';
		print '<TR BGCOLOR="blue"><TH>Rank<TH>Times<BR>executed<TH>Query</TH>';
	
		# printing the array in html formatted manner
		map { print "<tr><td>" . $i++ . "<td>$PQAGlobals::query_details{$_}->[0]<td>$_</tr>"; }  @sorted;

		print "</TABLE>";
	}
	
	return 1;
}	# ----------  end of subroutine PrintMostFreqQueries  ----------


#===  FUNCTION  ================================================================
#         NAME:  QueriesTookMostTime
#      PURPOSE:  ????
#   PARAMETERS:  nothing
#      RETURNS:  ????
#  DESCRIPTION:  ????
#       THROWS:  no exceptions
#     COMMENTS:  none
#     SEE ALSO:  n/a
#===============================================================================
sub QueriesTookMostTime {
	my @sorted;
	my $n_to_show;
	my $i=1;

	# sort the hash key based up on the total duration in it., and return it into array.
		# hashkey => [ total_count, ...., total_duration ]
	@sorted = sort { $PQAGlobals::query_details{$b}->[4] <=> $PQAGlobals::query_details{$a}->[4] } keys %PQAGlobals::query_details;

	# remove the unwanted elements from the array, i.e show only the n number of elements.
	# if n is not given, then default is top 20.
	$n_to_show = $PQAGlobals::options{"n"};
	splice(@sorted, $n_to_show)	if scalar(@sorted) > $n_to_show;

	if ($PQAGlobals::options{t})  {
		# print text report.
		map { print sprintf("%.3f", $PQAGlobals::query_details{$_}->[4]/1000) .  "\t$PQAGlobals::query_details{$_}->[0]\t$_\n"; }  @sorted;
	} else  {
		# print html report.

		print "<BR><H3>Queries Took Most Time</H3>";
		print '<TABLE BORDER="0">';
		print '<TR BGCOLOR="blue"><TH>Rank<TH>Total time<TH>Times<BR>executed<TH>Query</TH>';

		map { print "<tr><td>" . $i++ . "<td>" . sprintf("%.3f", $PQAGlobals::query_details{$_}->[4]/1000) .  "<td>$PQAGlobals::query_details{$_}->[0]<td>$_</tr>"; }  @sorted;

		print "</TABLE>";
	}
	
	return 1;
}	# ----------  end of subroutine QueriesTookMostTime  ----------


#===  FUNCTION  ================================================================
#         NAME:  FastestQueries
#      PURPOSE:  ????
#   PARAMETERS:  ????
#      RETURNS:  ????
#  DESCRIPTION:  ????
#       THROWS:  no exceptions
#     COMMENTS:  none
#     SEE ALSO:  n/a
#===============================================================================

sub FastestQueries {
	my @sorted;
	my $n_to_show;
	my $i = 1;


	# sort the hash key based up on the max duration in it., and return it into array.
		# hashkey => [ total_count, ...., max_duration ]
											# FIXME -> if same query has different variations in
											# time, and it has a chance to come in slowest queries will not come.
	@sorted = sort { $PQAGlobals::query_details{$a}->[2] <=> $PQAGlobals::query_details{$b}->[2] } keys %PQAGlobals::query_details;

	# remove the unwanted elements from the array, i.e show only the n number of elements.
	# if n is not given, then default is top 20.
	$n_to_show = $PQAGlobals::options{"n"};
 	splice(@sorted, $n_to_show)	if scalar(@sorted) > $n_to_show;
	                                                                                                                                           
	if ($PQAGlobals::options{t})  {
		# print text report.
		map { print sprintf("%.3f", $PQAGlobals::query_details{$_}->[2]/1000) .  "\t$_\n"; }  @sorted;
		
	} else  {
		# print html report.
		print "<BR><H3>Fastest Queries</H3>";
		print '<TABLE BORDER="0">';
		print '<TR BGCOLOR="blue"><TH>Rank<TH>Time<TH>Query</TH>';

		map { print "<tr><td>" . $i++ . "<td>" . sprintf("%.3f", $PQAGlobals::query_details{$_}->[2]/1000) .  "<td>$_</tr>"; }  @sorted;
		print "</TABLE>";
	}
	
	return 1;
}	# ----------  end of subroutine SlowestQueries  ----------


#===  FUNCTION  ================================================================
#         NAME:  SlowestQueries
#      PURPOSE:  ????
#   PARAMETERS:  ????
#      RETURNS:  ????
#  DESCRIPTION:  ????
#       THROWS:  no exceptions
#     COMMENTS:  none
#     SEE ALSO:  n/a
#===============================================================================

sub SlowestQueries {
	my @sorted;
	my $n_to_show;
	my $i = 1;


	# sort the hash key based up on the max duration in it., and return it into array.
		# hashkey => [ total_count, ...., max_duration ]
											# FIXME -> if same query has different variations in
											# time, and it has a chance to come in slowest queries will not come.
	@sorted = sort { $PQAGlobals::query_details{$b}->[3] <=> $PQAGlobals::query_details{$a}->[3] } keys %PQAGlobals::query_details;

	# remove the unwanted elements from the array, i.e show only the n number of elements.
	# if n is not given, then default is top 20.
	$n_to_show = $PQAGlobals::options{"n"};
 	splice(@sorted, $n_to_show)	if scalar(@sorted) > $n_to_show;
	                                                                                                                                           
	if ($PQAGlobals::options{t})  {
		# print text report.
		map { print sprintf("%.3f", $PQAGlobals::query_details{$_}->[3]/1000) .  "\t$_\n"; }  @sorted;
		
	} else  {
		# print html report.
		print "<BR><H3>Slowest Queries</H3>";
		print '<TABLE BORDER="0">';
		print '<TR BGCOLOR="blue"><TH>Rank<TH>Time<TH>Query</TH>';

		map { print "<tr><td>" . $i++ . "<td>" . sprintf("%.3f", $PQAGlobals::query_details{$_}->[3]/1000) .  "<td>$_</tr>"; }  @sorted;
		print "</TABLE>";
	}
	
	return 1;
}	# ----------  end of subroutine SlowestQueries  ----------




#===  FUNCTION  ================================================================
#         NAME:  startreport, endreport
#      PURPOSE:  to put the html tags
#   PARAMETERS:  ????
#      RETURNS:  returns true.
#  DESCRIPTION:  To print the required html tags, to form the html report.
#       THROWS:  no exceptions
#     COMMENTS:  none
#     SEE ALSO:  n/a
#===============================================================================


sub startreport {
	my $total_number_of_queries=0;
	my $unique_number_of_queries=0;
	my $shortest_query;
	my $longest_query;
	my @sorted;

	# number of keys is the number of unique queries.
 	$unique_number_of_queries=scalar keys %PQAGlobals::query_details;

	# total number of queries is the "total of counters of all queries".
	$total_number_of_queries += $PQAGlobals::query_details{$_}->[0] for ( keys %PQAGlobals::query_details );


	#---------------------------------------------------------------------------
	# finding the shortest, and longest query
	# sort the hash based upon the length of its key in ascending, and store it in hash.,
	# finally 0th element is shortest query, and last element is longest query.
	#---------------------------------------------------------------------------
					# FIXME : simplify this better.
		#@sorted = (sort { length($a) cmp length($b) } keys %PQAGlobals::query_details);
		#$shortest_query = $sorted[0];
		#$longest_query = $sorted[-1];
	# NOT YET TESTED................................................................ NOT tested.
	($shortest_query, $longest_query) = (sort { length($a) cmp length($b) } keys %PQAGlobals::query_details)[0,-1];

	#use Data::Dumper;
	#print Dumper(%PQAGlobals::query_details);

	if ( $PQAGlobals::options{"t"} )  {
		print "text report is not yet implemented, dont give -t in this version";	
	} else {
		print "<HTML><HEAD><TITLE>ePQA; Query analysis Report</TITLE></HEAD><BODY>";

		
		print "<style type='text/css'>
		body { background-color:white; 	font-family:Courier, Arial, Helvetica, sans-serif; font-size:14px;}
		h2 { text-align:center; }
		h3 { color:blue }
		p, td, th { font-family:Courier, Arial, Helvetica, sans-serif; font-size:14px; }
		th { color:white; background-color:#7B8CBE; }
		span.keyword { color:blue; }
		</style>";

		print "<CENTER><H1>Query analysis report</H1></CENTER>";

		print "<BR><H3>Overall Statistics</H3>";
		print "Total number of queries: $total_number_of_queries<BR>";
		print "Unique number of queries: $unique_number_of_queries<BR>";
		print "Shortest query: $shortest_query<BR>";
		print "Longest query: $longest_query<BR>";
		print "Log file parsed in: $PQAGlobals::LogFileParsedTime seconds<BR><BR><BR>";
	}
	
	return 1;
}	# ----------  end of subroutine starthtml  ----------


sub endreport {
	if ( $PQAGlobals::options{"t"} )  {
	} else {
		print "<BR><BR><BR>Report generated by <a href='http://epqa.sourceforge.net/'>epqa</a></BODY></HTML>";
	}
	
	return 1;
}	# ----------  end of subroutine endhtml  ----------


#===  FUNCTION  ================================================================
#         NAME:  PrintErrorQuery
#      PURPOSE:  To print the error queries.,
#   PARAMETERS:  nothing
#      RETURNS:  ????
#  DESCRIPTION:  
#  			Print the error queries to the report from the hash.,
#       THROWS:  no exceptions
#     COMMENTS:  none
#     SEE ALSO:  n/a
#===============================================================================

sub PrintErrorQuery {

	if ( $PQAGlobals::options{"t"} )  {
		
	} else {
		print "<BR><BR>Error Queries<BR>";
		print '<TABLE BORDER="0">';
		print '<TR BGCOLOR="blue"><TH>Number of Times<TH>Query</TH>';
		map { print "<tr><td>$PQAGlobals::error_query_details{$_}<td>$_</tr>" } sort { $PQAGlobals::error_query_details{$b} <=> $PQAGlobals::error_query_details{$a} } keys %PQAGlobals::error_query_details;
		print '</TABLE>';
	}

	
}	# ----------  end of subroutine PrintErrorQuery  ----------


1;
